class ProcessorResponseTypes(object):
    """
    A set of constants representing processor response types.
    """

    Approved = "approved"
    SoftDeclined = "soft_declined"
    HardDeclined = "hard_declined"
